@@ -43,7 +43,8 @@ typedef enum
 {
   GST_GL_DOWNLOAD_MODE_PASSTHROUGH,
   GST_GL_DOWNLOAD_MODE_PBO_TRANSFERS,
-  GST_GL_DOWNLOAD_MODE_DMABUF_EXPORTS
+  GST_GL_DOWNLOAD_MODE_DMABUF_EXPORTS,
+  GST_GL_DOWNLOAD_MODE_NVMM,
 } GstGlDownloadMode;
 
 struct _GstGLDownloadElement
