@@ -183,6 +183,9 @@ gst_stream_splitter_sink_event (GstPad * pad, GstObject * parent,
       GST_EVENT_TYPE_NAME (event));
 
   switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_STREAM_START:
+      toall = TRUE;
+      break;
     case GST_EVENT_CAPS:
     {
       GstCaps *caps;
@@ -415,6 +418,7 @@ gst_stream_splitter_sink_query (GstPad * pad, GstObject * parent,
 static gboolean
 gst_stream_splitter_sink_setcaps (GstPad * pad, GstCaps * caps)
 {
+  GstPad *prev = NULL;
   GstStreamSplitter *stream_splitter =
       (GstStreamSplitter *) GST_PAD_PARENT (pad);
   guint32 cookie;
@@ -433,6 +437,9 @@ resync:
   }
 
   res = FALSE;
+  prev =
+      stream_splitter->
+      current ? gst_object_ref (stream_splitter->current) : NULL;
   tmp = stream_splitter->srcpads;
   cookie = stream_splitter->cookie;
 
@@ -440,6 +447,7 @@ resync:
     GstPad *srcpad = (GstPad *) tmp->data;
     GstCaps *peercaps;
 
+    // GST_ERROR_OBJECT (srcpad, "Checking");
     STREAMS_UNLOCK (stream_splitter);
     peercaps = gst_pad_peer_query_caps (srcpad, NULL);
     if (peercaps) {
@@ -452,15 +460,31 @@ resync:
       goto resync;
 
     if (res) {
-      /* FIXME : we need to switch properly */
       GST_DEBUG_OBJECT (srcpad, "Setting caps on this pad was successful");
       stream_splitter->current = srcpad;
+      if (prev && prev != srcpad) {
+        /* When switching branches, we need to ensure that the branch it was
+         * pushing to before is drainned and the encoder pushes all its internal
+         * data, we do it by first letting downstream streamcombiner that we are
+         * working on it, and then send an EOS to ensure all the data is
+         * processed and cleanup the encoder with a flush start/flush stop
+         * dance. Those events are discarded in the stream combiner */
+        /* FIXME We should theoretically be able to use a DRAIN query here
+         * instead but the encoders do not react properly to them */
+        gst_pad_push_event (prev,
+            gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM,
+                gst_structure_new_empty ("start-draining-encoder")));
+        gst_pad_push_event (prev, gst_event_new_eos ());
+        gst_pad_push_event (prev, gst_event_new_flush_start ());
+        gst_pad_push_event (prev, gst_event_new_flush_stop (FALSE));
+      }
       goto beach;
     }
     tmp = tmp->next;
   }
 
 beach:
+  gst_clear_object (&prev);
   STREAMS_UNLOCK (stream_splitter);
   return res;
 }
